#include <db.h>
#include <stdio.h>



typedef struct _AFFILE {
    /* For Raw files */
    FILE *raw;				// if it is a raw file

    /* For AFF Segment Files */
    FILE *aseg;

    /* Image information */
    unsigned long    segsize;	  // the size of segments in this file
    unsigned long long imagesize;			// how big is the disk image
    unsigned char badflag[512];		// bad sector flag

    /* Implement a stream abstraction */
    int     segnum;			// -1 means no segment loaded
    unsigned char *segbuf;		// where the data is
    size_t   segbytes;			// number of bytes that were loaded
    unsigned long long pos;				// location in stream
} AFFILE;

/* The header for an AFF file. All binary numbers are stored in network byte order. */
struct af_head {
    char header[8];			// "AFF10\r\n\000"
    /* segments follow */
};

#define AF_HEADER "AFF10\r\n\000"

struct af_segment_head {
    char flag[4];			// "AFF\000"
    unsigned long name_len;		// length of segment name
    unsigned long data_len;		// length of segment data, if any
    unsigned long arg;			// argument for name;
    /* name follows, then data */
};

struct af_segment_tail {
    char flag[4];			// "ATT\000"
    unsigned long segment_len;	 // includes head, tail, name & length
};

#define AF_SEGHEAD "AFF\000"
#define AF_SEGTAIL "ATT\000"


/* Utility Functions */

#ifdef __cplusplus
extern "C" {
#endif

const char *af_hexbuf(char *dst,const unsigned char *bin,int bytes);

/* Low-level functions for putting */

void af_put_segv(AFFILE *af,const char *name,unsigned long arg,const void *value,int vallen);
void af_put_segq(AFFILE *af,const char *name,long long value);
void af_put_badflag(AFFILE *af);


/* probe the next segment.
 * Returns - segname - the name of the next segment (must be freed)
 *         - segsize - number of bytes the entire segment is.
 * doesn't change af->aseg pointer.
 */
int af_probe_next_segment(AFFILE *af,char **segname,unsigned long *datasize,unsigned long *segsize);

/* get functions */
int af_get_next_segv(AFFILE *af,char **name,unsigned long *arg,
		   unsigned char **data,size_t *datalen);
int af_get_segv(AFFILE *af,const char *name,unsigned long *arg,
		   unsigned char **data,size_t *datalen);
int af_get_image_segment(AFFILE *af,int segnum,unsigned char **data,size_t *bytes);

int af_get_segq(AFFILE *af,const char *name,unsigned long long *quad);
void af_get_segsize(AFFILE *af);			// sets up internal values

/* update functions */
int af_update_segv(AFFILE *af,const char *name,unsigned long arg,const void *value,int vallen);
int af_update_segq(AFFILE *af,const char *name,unsigned long long quad);

/* af_file stream functions */
AFFILE *af_open(const char *filename,int flags,int mode);
int	af_eof(AFFILE *af);
int	af_close(AFFILE *af);
int	af_purge(AFFILE *af);
int	af_read(AFFILE *af,unsigned char *buf,int count);
fpos_t  af_seek(AFFILE *af,fpos_t pos,int whence);
fpos_t  af_tell(AFFILE *af);
int	af_rewind(AFFILE *af);
int	af_backspace(AFFILE *af);	// back up one segment


/* aff creation functions */
void af_setsize(AFFILE *af,long segsize,unsigned long long imagesize);

#ifdef __cplusplus
}
#endif





#define AF_IGNORE "ignore"
#define AF_SEGSIZE "segsize"
#define AF_IMAGESIZE "imagesize"
#define AF_BADSECTORS "badsectors"
#define AF_BADFLAG "badflag"
#define AF_SEG "seg%d"			// arg=1 means it is compressed with zlib
#define AF_SEG_COMPRESSED 0x0001
#define AF_MD5 "md5"
#define AF_SHA1 "sha1"
